 /////////////////////////////////////////////////////////////////////////
//																		//
// Filename	: LA.h														//
// Project	: LA (Linear Algebra library)								//
// Author	: Iain Hutchison (iain.hutchison@st-annes.ox.ac.uk)			//
//																		//
//////////////////////////////////////////////////////////////////////////
//																		//
// Notes	:															//
//																		//
/////////////////////////////////////////////////////////////////////////

#ifndef __LA_H__
#define __LA_H__

// forward declarations

class Vector4;
class Point4;
class Matrix44;

 /////////////////////
//  class : Point4  //
/////////////////////

class Point4 {
public:
	float	x,y,z,w;
public:
	Point4();
	Point4(float nx,float ny,float nz,float nw);
	Point4(const Point4 &p);
	Point4(const Vector4 &v);
public:
	Point4	operator=(Point4 p);
	int		operator==(Point4 p);
	int		operator!=(Point4 p);
	Point4	operator+=(Vector4 v);
	Point4	operator-=(Vector4 v);
	Point4	operator+(Vector4 v);
	Point4	operator-(Vector4 v);
	Vector4	operator-(Point4 p);
	float	operator[](int index);
};

 //////////////////////
//  class : Vector4  //
//////////////////////

class Vector4 {
public:
	float	i,j,k,h;
public:
	Vector4();
	Vector4(float ni,float nj,float nk,float nh);
	Vector4(const Vector4 &v);
	Vector4(const Point4 &p);
public:
	Vector4	operator=(Vector4 v);
	int		operator==(Vector4 v);
	int		operator!=(Vector4 v);
	Vector4	operator+=(Vector4 v);
	Vector4	operator-=(Vector4 v);
	Vector4	operator*=(float f);
	Point4	operator+(Point4 p);
	Vector4	operator+(Vector4 v);
	Vector4	operator-(Vector4 v);
	Vector4	operator*(float f);
	Vector4	operator-();
	float	dotProduct(Vector4 v);
	float	dotProduct(Point4 p);
	Vector4	crossProduct(Vector4 v);
	float	modulus();
	float	modulusSquared();
	void	makeUnit();
	void	homogenise();
	float&	operator[](int index);
};

 ///////////////////////
//  class : Matrix44  //
///////////////////////

class Matrix44 {
public:
	Vector4	rows[4];
public:
	Matrix44();
	Matrix44(const Matrix44 &m);
public:
	Matrix44	operator=(Matrix44 m);
	int			operator==(Matrix44 &m);
	int			operator!=(Matrix44 &m);
	Matrix44	operator+=(Matrix44 &m);
	Matrix44	operator-=(Matrix44 &m);
	Matrix44	operator*=(Matrix44 &b);
	Matrix44	operator*=(float f);
	Vector4&	operator[](int i);
	Matrix44	operator*(Matrix44 &b);
	Matrix44	operator*(float f);
	Vector4 	operator*(Vector4 v);
	Point4	 	operator*(Point4 p);
	void		transpose();
};


#endif

 /////////////////////////////////////////////////////////////////////////
//															end of file	//
/////////////////////////////////////////////////////////////////////////
